﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            Width      = 800;
            Height     = 600;
            Title      = "시스템 매개 변수 조회하기";
            FontFamily = new FontFamily("나눔고딕코딩");
            FontSize   = 16;

            ListView listView = new ListView();

            Content = listView;

            GridView gridView = new GridView();

            listView.View = gridView;

            GridViewColumn nameGridViewColumn = new GridViewColumn();

            nameGridViewColumn.Header               = "Property Name";
            nameGridViewColumn.Width                = 200;
            nameGridViewColumn.DisplayMemberBinding = new Binding("Name");

            gridView.Columns.Add(nameGridViewColumn);

            GridViewColumn valueViewColumn = new GridViewColumn();

            valueViewColumn.Header               = "Value";
            valueViewColumn.Width                = 200;
            valueViewColumn.DisplayMemberBinding = new Binding("Value");

            gridView.Columns.Add(valueViewColumn);

            PropertyInfo[] propertyInfoArray = typeof(SystemParameters).GetProperties();

            #region 방법 1

            Array.Sort(propertyInfoArray, new PropertyInfoComparer());
            
            foreach(PropertyInfo propertyInfo in propertyInfoArray)
            {
                if(propertyInfo.PropertyType != typeof(ResourceKey))
                {
                    SystemParam systemParam = new SystemParam();
            
                    systemParam.Name  = propertyInfo.Name;
                    systemParam.Value = propertyInfo.GetValue(null, null);
            
                    listView.Items.Add(systemParam);
                }
            }

            #endregion
            #region 방법 2

            //foreach(PropertyInfo propertyInfo in propertyInfoArray)
            //{
            //    if(propertyInfo.PropertyType != typeof(ResourceKey))
            //    {
            //        SystemParam systemParam = new SystemParam();
            //
            //        systemParam.Name  = propertyInfo.Name;
            //        systemParam.Value = propertyInfo.GetValue(null, null);
            //
            //        listView.Items.Add(systemParam);
            //    }
            //}
            //
            //listView.Items.SortDescriptions.Add(new SortDescription("Name", ListSortDirection.Ascending));

            #endregion
            #region 방법 3

            //SortedList<string, SystemParam> sortedList = new SortedList<string, SystemParam>();
            //
            //foreach(PropertyInfo propertyInfo in propertyInfoArray)
            //{
            //    if(propertyInfo.PropertyType != typeof(ResourceKey))
            //    {
            //        SystemParam systemParam = new SystemParam();
            //
            //        systemParam.Name  = propertyInfo.Name;
            //        systemParam.Value = propertyInfo.GetValue(null, null);
            //
            //        sortedList.Add(propertyInfo.Name, systemParam);
            //    }
            //}
            //
            //listView.ItemsSource = sortedList.Values;

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        [STAThread]
        public static void Main()
        {
            Application application = new Application();

            application.Run(new MainWindow());
        }

        #endregion
    }
}